<?php
namespace FM\Controller;

use FM\Controller\FMCrud;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use FM\Service\SetoresService;
use FM\Traits\TResponse;

class SetoresController extends FMCrud {
    
    use TResponse;
    protected $dataSourceName = 'fcm_checklistagend';
    protected $SetoresService;
   
    public function __construct($crudService, $environment, SetoresService $SetoresService) {
        parent::__construct($crudService, $environment);
        $this->SetoresService = $SetoresService;
    }

    public function getSectorsController(DTO\Request $request, DTO\Response $response) {
        try {
            $data = $this->SetoresService->getSectorsService();
            $this->sendResponse($data, 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }
    
    public function sectorsListController(DTO\Request $request, DTO\Response $response) {
        $var = $this->SetoresService->sectorsListService();
        $dtsVal = new DataSet('info', $var);
        $response->addDataSet($dtsVal);
    }
    
    public function createsectorController(DTO\Request $request, DTO\Response $response) {
        
        try {
            $params = $request->getParameters();

            $nmsetor = $params['sectorInfos']['NMSETOR'];
            $nrfuncresponsavel = $params['sectorInfos']['NRFUNCRESPONSAVEL'];
        
            $data = $this->SetoresService->createSectorService($nmsetor, $nrfuncresponsavel);
            $this->sendResponse($data, 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }
    
    public function getSectorInfosController(DTO\Request $request, DTO\Response $response) {
        $params = $request->getParameters();
        
        $nrsetor = $params['params']['NRSETOR'];
        
        $var = $this->SetoresService->getSectorInfosService($nrsetor);
        $dtsVal = new DataSet('info', $var);
        $response->addDataSet($dtsVal);
    }
    
    public function updateSectorController(DTO\Request $request, DTO\Response $response) {
        
        try {
            $params = $request->getParameters();

            $nrsetor = $params['params']['NRSETOR'];
            $nmsetor = $params['params']['NMSETOR'];
            $nrfuncresponsavel = $params['params']['NRFUNCRESPONSAVEL'];
            
            $data = $this->SetoresService->updateSectorService($nrsetor, $nmsetor, $nrfuncresponsavel);
            $this->sendResponse($data, 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }
    
    public function deleteSectorsController(DTO\Request $request, DTO\Response $response) {

        try {
            $data = $this->SetoresService->deleteSectorsService($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }
    
    public function getLinkedLocalsController(DTO\Request $request, DTO\Response $response) {
        $params = $request->getParameters();
        
        $var = $this->SetoresService->getLinkedLocalsService($params["id"]);
        $dtsVal = new DataSet('info', $var);
        $response->addDataSet($dtsVal);
    }

}
?>