/* eslint-disable */
import {
  Metadata, IDictionary,
  Loader, Http, Router,
} from '@zeedhi/core';
import {
  Form, IComponentRender, CheckboxMultiple, Col, Checkbox,
  Button, Select, SelectMultiple, TextInput, Time, AlertService, Switch, Date as DateInput, DialogService, LoadingService,
} from '@zeedhi/common';
import { getRouteParams } from '@/router';
import { Utilities } from '@/util/Utilities';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class CadastroController {
  public modalController = Loader.getInstance('NovoCriterioController');

  public carregandoModal = Loader.getInstance('ModalLoading');

  public origem!: string;

  public setOrigem() {
    this.origem = (window.location.search.slice(1).split('=')[1]) ? '/Calendario' : '/FM_0000_000_tarefas';
  }

  public setDados(nome: string, cod: number) {
    return {
      name: nome,
      value: cod,
    };
  }

  public setAllSelects() {
    Http.get('/getTipoLocal').then((response: IDictionary) => {
      window.localStorage.setItem('JSONTIPOLOCAL', JSON.stringify(response.data.dataset.tipoLocal));
    });
    Http.get('/Local').then((response: IDictionary) => {
      window.localStorage.setItem('JSONLOCAL', JSON.stringify(response.data.dataset.Locals));
    });
    Http.get('/checklist').then((response: IDictionary) => {
      window.localStorage.setItem('JSONCHECKLIST', JSON.stringify(response.data.dataset.checklist));
    });
    Http.get('/itensChecklist').then((response: IDictionary) => {
      window.localStorage.setItem('JSONITENSCHECKLIST', JSON.stringify(response.data.dataset.itensChecklist));
    });
    Http.get('/escoposComAtividadeChk').then((response: IDictionary) => {
      window.localStorage.setItem('JSONESCOPOS', JSON.stringify(response.data.dataset.escopos));
    });
    Http.get('/getTarefas').then((response: IDictionary) => {
      window.localStorage.setItem('JSONTAREFA', JSON.stringify(response.data.dataset.task));
    });
    Http.get('/getCriterios').then((response: IComponentRender) => {
      window.localStorage.setItem('JSONCRITERIOS', JSON.stringify(response.data.dataset.criterio));
    });
    Http.get('/getFuncionario').then((response: IDictionary) => {
      window.localStorage.setItem('JSONFUNCIONARIOS', JSON.stringify(response.data.dataset.funcionario));
    });
    Http.get('/getSupervisor').then((response: IDictionary) => {
      window.localStorage.setItem('JSONSUPERVISORES', JSON.stringify(response.data.dataset.funcionario));
    });
  }

  public async setTipoLocal({ component }:any) {
    component.datasource.loading = true;
    const dados: any = JSON.parse(<string>window.localStorage.getItem('JSONTIPOLOCAL')) || [];
    const dadosForUpdate = (JSON.parse(<string>localStorage.getItem('dadosToEditTarefa'))) || [];
    if (dados.length > 0) {
      await component.datasource.updateData(dados);
    } else {
      await Http.get('/getTipoLocal').then((response: IDictionary) => {
        component.datasource.updateData(response.data.dataset.tipoLocal);
        window.localStorage.setItem('JSONTIPOLOCAL', JSON.stringify(response.data.dataset.tipoLocal));
      });
    }
    if ('NRTIPOLOCALCLI' in dadosForUpdate && component.name.includes('Edit')) {
      component.setValue([dadosForUpdate.NRTIPOLOCALCLI]);
      delete dadosForUpdate.NRTIPOLOCALCLI;
      window.localStorage.setItem('dadosToEditTarefa', JSON.stringify(dadosForUpdate));
    }
    component.datasource.loading = false;
  }

  public OriginalLocals: any = [];

  public async setLocal({ component }:any) {
    component.datasource.loading = true;
    const dados: any = JSON.parse(<string>window.localStorage.getItem('JSONLOCAL')) || [];
    if (dados.length > 0) {
      await component.datasource.updateData(dados);
      this.OriginalLocals = dados;
    } else {
      await Http.get('/Local').then((response: IDictionary) => {
        component.datasource.updateData(response.data.dataset.Locals);
        this.OriginalLocals = response.data.dataset.Locals;
        window.localStorage.setItem('JSONLOCAL', JSON.stringify(response.data.dataset.Locals));
      });
    }
    component.datasource.loading = false;
  }

  public OriginalChecklist: any = [];

  public async setChecklist({ component }:any) {
    component.datasource.loading = true;
    const dados: any = JSON.parse(<string>window.localStorage.getItem('JSONCHECKLIST')) || [];
    if (dados.length > 0) {
      await component.datasource.updateData(dados);
      this.OriginalChecklist = dados;
    } else {
      await Http.get('/checklist').then((response: IDictionary) => {
        component.datasource.updateData(response.data.dataset.checklist);
        this.OriginalChecklist = response.data.dataset.checklist;
        window.localStorage.setItem('JSONCHECKLIST', JSON.stringify(response.data.dataset.checklist));
      });
    }
    component.datasource.loading = false;
  }

  public async setValuesLocalAndChecklist({ component }: { component:SelectMultiple }) {
    const dadosForUpdate = JSON.parse(<string>localStorage.getItem('dadosToEditTarefa')) || {};
    const adicional: string = (component.name.includes('Edit')) ? 'Edit' : '';
    const local = Metadata.getInstance<SelectMultiple>(`${adicional}selectNomeLocal`);
    const checklist = Metadata.getInstance<Select>(`${adicional}selectChecklist`);
    let valoresLocal: any = [];
    let valoreschecklist: any = [];
    local.datasource.search = '';
    checklist.datasource.search = '';
    if (component.value && component.value !== null && component.value.length > 0) {
      valoresLocal = (local.value || []);
      const novoDadosLocal: any = this.OriginalLocals.filter((elemento: any) => component.value.includes(elemento.NRTIPOLOCALCLI));
      valoresLocal = novoDadosLocal.filter((elemento: any) => valoresLocal.includes(elemento.NRLOCALCLIENTE));
      if (component.name.includes('Edit')) {
        if ('NRLOCALCLIENTE' in dadosForUpdate) {
          valoresLocal = [dadosForUpdate.NRLOCALCLIENTE];
          delete dadosForUpdate.NRLOCALCLIENTE;
        }
      }
      local.datasource.updateData(novoDadosLocal);
      valoreschecklist = (checklist.value || null);
      const novoDadosChecklist: any = this.OriginalChecklist.filter((elemento: any) => component.value.includes(elemento.NRTIPOLOCALCLI));
      valoreschecklist = novoDadosChecklist.filter((elemento: any) => valoreschecklist === elemento.NRCHECKLIST);
      valoreschecklist = (valoreschecklist.length > 0) ? valoreschecklist : [];
      if (component.name.includes('Edit')) {
        if ('NRCHECKLIST' in dadosForUpdate) {
          valoreschecklist = dadosForUpdate.NRCHECKLIST[0] || null;
          delete dadosForUpdate.NRCHECKLIST;
        }
      }
      checklist.datasource.updateData(novoDadosChecklist);
      local.disabled = false;
      checklist.disabled = false;
    } else {
      local.disabled = true;
      checklist.disabled = true;
      checklist.search = '';
    }
    window.localStorage.setItem('dadosToEditTarefa', JSON.stringify(dadosForUpdate));
    local.setValue((valoresLocal.length > 0) ? valoresLocal : null);
    checklist.setValue(valoreschecklist);
  }

  public OriginalItensChecklist: any = [];

  public async setItensChecklist({ component }:any) {
    component.datasource.loading = true;
    const dados: any = JSON.parse(<string>window.localStorage.getItem('JSONITENSCHECKLIST')) || [];
    if (dados.length > 0) {
      await component.datasource.updateData(dados);
      this.OriginalItensChecklist = dados;
    } else {
      await Http.get('/itensChecklist').then((response: IDictionary) => {
        component.datasource.updateData(response.data.dataset.itensChecklist);
        this.OriginalItensChecklist = response.data.dataset.itensChecklist;
        window.localStorage.setItem('JSONITENSCHECKLIST', JSON.stringify(response.data.dataset.itensChecklist));
      });
    }
    component.datasource.loading = false;
  }

  public getItensChecklist({ component }:any) {
    const dadosForUpdate = JSON.parse(<string>localStorage.getItem('dadosToEditTarefa')) || {};
    const adicional: string = (component.name.includes('Edit')) ? 'Edit' : '';
    const itensChecklist = Metadata.getInstance<SelectMultiple>(`${adicional}selectTipoTarefa`);
    let valoresItensChecklist: any = [];
    if (component.value && component.value !== null) {
      valoresItensChecklist = (itensChecklist.value || []);
      const novoItensChecklist: any = this.OriginalItensChecklist.filter((elemento: any) => component.value === elemento.NRCHECKLIST);
      valoresItensChecklist = novoItensChecklist.filter((elemento: any) => valoresItensChecklist.includes(elemento.NRATIVIDADECHK));
      if (component.name.includes('Edit')) {
        if ('NRATIVIDADECHK' in dadosForUpdate) {
          valoresItensChecklist = dadosForUpdate.NRATIVIDADECHK || [];
          delete dadosForUpdate.NRATIVIDADECHK;
          window.localStorage.setItem('dadosToEditTarefa', JSON.stringify(dadosForUpdate));
        }
      }
      itensChecklist.datasource.updateData(novoItensChecklist);
      itensChecklist.disabled = false;
    } else {
      itensChecklist.disabled = true;
    }
    itensChecklist.setValue((valoresItensChecklist.length > 0) ? valoresItensChecklist : null);
  }

  public OriginalEscopo: any = [];

  public async setEscopos({ component }:any) {
    component.datasource.loading = true;
    const dados: any = JSON.parse(<string>window.localStorage.getItem('JSONESCOPOS')) || [];
    if (dados.length > 0) {
      await component.datasource.updateData(dados);
      this.OriginalEscopo = dados;
    } else {
      await Http.get('/escoposComAtividadeChk').then((response: IDictionary) => {
        component.datasource.updateData(response.data.dataset.escopos);
        this.OriginalEscopo = response.data.dataset.escopos;
        window.localStorage.setItem('JSONESCOPOS', JSON.stringify(response.data.dataset.escopos));
      });
    }
    component.datasource.loading = false;
  }

  public async getEscopos({ component }:any) {
    const dadosForUpdate = JSON.parse(<string>localStorage.getItem('dadosToEditTarefa')) || {};
    const adicional: string = (component.name.includes('Edit')) ? 'Edit' : '';
    const escopos = Metadata.getInstance<SelectMultiple>(`${adicional}selectEscopoServico`);
    let valoresEscopo: any = [];
    if (component.value && component.value !== null && component.value.length > 0) {
      valoresEscopo = (escopos.value || []);
      const novoEscopo: any = this.OriginalEscopo.filter((elemento: any) => component.value.includes(elemento.NRATIVIDADECHK));
      if ((this.OriginalTarefas.filter((elemento: any) => component.value.includes(elemento.NRATIVIDADECHK))).map((elemento: any) => elemento.NRESCOPOSTAREFA).includes(null)) {
        novoEscopo.push({
          NMESCOPOSTAREFA: ' ',
          NRESCOPOSTAREFA: null,
        });
      }
      valoresEscopo = novoEscopo.filter((elemento: any) => valoresEscopo.includes(elemento.NRATIVIDADECHK));
      escopos.datasource.updateData(novoEscopo);
      escopos.disabled = false;
    } else {
      escopos.disabled = true;
    }
    if (component.name.includes('Edit')) {
      if ('NRESCOPOSTAREFA' in dadosForUpdate) {
        escopos.setValue(dadosForUpdate.NRESCOPOSTAREFA || []);
        delete dadosForUpdate.NRESCOPOSTAREFA;
        window.localStorage.setItem('dadosToEditTarefa', JSON.stringify(dadosForUpdate));
      }
    } else {
      await escopos.setValue((valoresEscopo.length > 0) ? valoresEscopo : null);
    }
  }

  public OriginalTarefas: any = [];

  public async setTarefas({ component }:any) {
    component.datasource.loading = true;
    const dados: any = JSON.parse(<string>window.localStorage.getItem('JSONTAREFA')) || [];
    if (dados.length > 0) {
      await component.datasource.updateData(dados);
      this.OriginalTarefas = dados;
    } else {
      await Http.get('/getTarefas').then((response: IDictionary) => {
        component.datasource.updateData(response.data.dataset.task);
        this.OriginalTarefas = response.data.dataset.task;
        window.localStorage.setItem('JSONTAREFA', JSON.stringify(response.data.dataset.task));
      });
    }
    component.datasource.loading = false;
  }

  public async getTarefas({ component }:any) {
    const dadosForUpdate = JSON.parse(<string>localStorage.getItem('dadosToEditTarefa')) || {};
    const adicional: string = (component.name.includes('Edit')) ? 'Edit' : '';
    const tipoTarefa = Metadata.getInstance<SelectMultiple>(`${adicional}selectTipoTarefa`);
    const tarefas = Metadata.getInstance<SelectMultiple>(`${adicional}selectTarefa`);
    let valoresTarefa: any = [];
    if (component.value && component.value !== null && component.value.length > 0) {
      valoresTarefa = (tarefas.value || []);
      const novaTarefas: any = this.OriginalTarefas.filter((elemento: any) => component.value.includes(elemento.NRESCOPOSTAREFA)  && tipoTarefa.value.includes(elemento.NRATIVIDADECHK));
      valoresTarefa = novaTarefas.filter((elemento: any) => valoresTarefa.includes(elemento.NRESCOPOSTAREFA));
      tarefas.datasource.updateData(novaTarefas);
      tarefas.disabled = false;
    } else {
      tarefas.disabled = true;
    }
    if (component.name.includes('Edit')) {
      if ('NRTAREFACHK' in dadosForUpdate) {
        tarefas.setValue(dadosForUpdate.NRTAREFACHK || []);
        delete dadosForUpdate.NRTAREFACHK;
        window.localStorage.setItem('dadosToEditTarefa', JSON.stringify(dadosForUpdate));
      }
    } else {
      await  tarefas.setValue((valoresTarefa.length > 0) ? valoresTarefa : null);
    }
  }

  public OriginalCriterios: any = [];

  public async getCriteriosDeAvaliacao({ component }:any) {
    const dados: any = JSON.parse(<string>window.localStorage.getItem('JSONCRITERIOS')) || [];
    if (dados.length > 0) {
      this.OriginalCriterios = dados;
    } else {
      await Http.get('/getCriterios').then((response: IComponentRender) => {
        this.OriginalCriterios = response.data.dataset.criterio;
        window.localStorage.setItem('JSONCRITERIOS', JSON.stringify(response.data.dataset.criterio));
      });
    }
    if (component.component !== 'ZdCol') {
      await component.datasource.updateData(dados);
    }
  }

  public async setCriterio({ component }: { component:Select }) {
    const dadosForUpdate = JSON.parse(<string>localStorage.getItem('dadosToEditTarefa')) || {};
    const adicional: string = (component.name.includes('Edit')) ? 'Edit' : '';
    const checklist = Metadata.getInstance<Select>(`${adicional}selectChecklist`).value || [];
    const itensChecklist = Metadata.getInstance<SelectMultiple>(`${adicional}selectTipoTarefa`).value || [];
    const tarefas = Metadata.getInstance<SelectMultiple>(`${adicional}selectTarefa`).value || [];
    const criterios: any = (!component.name.includes('Edit')) ? Metadata.getInstance<CheckboxMultiple>(`CheckBoxDeCriterios`) : Metadata.getInstance<SelectMultiple>(`EditCriterios`);
    const btnNovoCriterios = Metadata.getInstance<Button>('BotaoNovoCriterio');
    if (component.value !== null && component.value.length) {
      const newDados = this.OriginalCriterios.filter((elemento: any) => checklist === elemento.NRCHECKLIST && itensChecklist.includes(elemento.NRATIVIDADECHK) && tarefas.includes(elemento.NRTAREFACHK));
       const newDadosFilter = newDados.filter((elemento: any, pos: number) => newDados.findIndex((unidade: any) => unidade.DSCRITERIOTAREFA === elemento.DSCRITERIOTAREFA) === pos);
       newDadosFilter.forEach((elemento: any) => {
         elemento.NRCRITERIOTAREFACHK = (newDados.filter((unidade: any) => unidade.DSCRITERIOTAREFA === elemento.DSCRITERIOTAREFA).map((unidade: any) => unidade.NRCRITERIOTAREFACHK)).toString();
       });
      if (criterios.component === 'ZdCheckboxMultiple') {
        criterios.isVisible = true;
        btnNovoCriterios.disabled = false;
      } else if (criterios.component === 'ZdSelectMultiple') {
        criterios.disabled = false;
      }
      criterios.datasource!.updateData(newDadosFilter);
      if (component.name.includes('Edit')) {
        if ('NRCRITERIOTAREFA' in dadosForUpdate) {
          criterios.setValue(dadosForUpdate.NRCRITERIOTAREFA || []);
          delete dadosForUpdate.NRCRITERIOTAREFA;
          window.localStorage.setItem('dadosToEditTarefa', JSON.stringify(dadosForUpdate));
        }
      }
    } else {
      if (criterios.component === 'ZdCheckboxMultiple') {
        criterios.isVisible = false;
        btnNovoCriterios.disabled = true;
      } else if (criterios.component === 'ZdSelectMultiple') {
        criterios.disabled = true;
        criterios.setValue([]);
      }
    }
  }

  public async setFuncionario({ component }:any) {
    const dadosForUpdate = JSON.parse(<string>localStorage.getItem('dadosToEditTarefa')) || {};
    const adicional: string = (component.name.includes('Edit')) ? 'Edit' : '';
    component.datasource.loading = true;
    const dados: any = JSON.parse(<string>window.localStorage.getItem('JSONFUNCIONARIOS')) || [];
    if (dados.length > 0) {
      await component.datasource.updateData(dados);
    } else {
      await Http.get('/getFuncionario').then((response: IDictionary) => {
        component.datasource.updateData(response.data.dataset.funcionario);
        window.localStorage.setItem('JSONFUNCIONARIOS', JSON.stringify(response.data.dataset.funcionario));
      });
    }
    if (component.name.includes('Edit')) {
        if ('NRFUNCIONARIO' in dadosForUpdate) {
          component.setValue(dadosForUpdate.NRFUNCIONARIO || null);
          delete dadosForUpdate.NRFUNCIONARIO;
          window.localStorage.setItem('dadosToEditTarefa', JSON.stringify(dadosForUpdate));
        }
      }
    component.datasource.loading = false;
  }

  public async setSupervisor({ component }:any) {
    const dadosForUpdate = JSON.parse(<string>localStorage.getItem('dadosToEditTarefa')) || {};
    const adicional: string = (component.name.includes('Edit')) ? 'Edit' : '';
    component.datasource.loading = true;
    const dados: any = JSON.parse(<string>window.localStorage.getItem('JSONSUPERVISORES')) || [];
    if (dados.length > 0) {
      await component.datasource.updateData(dados);
    } else {
      await Http.get('/getSupervisor').then((response: IDictionary) => {
        component.datasource.updateData(response.data.dataset.funcionario);
        window.localStorage.setItem('JSONSUPERVISORES', JSON.stringify(response.data.dataset.funcionario));
      });
    }
    if (component.name.includes('Edit')) {
        if ('NRENCARREGADO' in dadosForUpdate) {
          component.setValue(dadosForUpdate.NRENCARREGADO || null);
          delete dadosForUpdate.NRENCARREGADO;
          window.localStorage.setItem('dadosToEditTarefa', JSON.stringify(dadosForUpdate));
        }
      }
    component.datasource.loading = false;
  }


  // Comandos do modal
  public createModal() {
    this.modalController.createBasicUsageModal();
  }

  public destroyModal() {
    this.modalController.destroyBasicUsageModal();
  }

  public openModal() {
    this.modalController.openBasicModal();
  }

  public fecharModal() {
    this.modalController.closeBasicModal();
  }

  public modelCrterio(criterio: IDictionary) {
    return {
      name: 'criterioTarefa',
      component: 'ZdCheckbox',
      label: criterio.name,
      cssClass: 'zd-mx-3',
      value: criterio.value,
    };
  }

  public novoCriterio() {
    const form = Metadata.getInstance<Form>('modalCriacaoCriterio');
    if (form.validate()) {
      const chk = Metadata.getInstance<CheckboxMultiple>('CheckBoxDeCriterios');
      const valor = Metadata.getInstance<TextInput>('InputNovoCriterio').value;
      chk!.datasource!.data!.push({
        NRCRITERIOTAREFACHK: valor,
        DSCRITERIOTAREFA: valor,
      });
      chk!.value.push({
        NRCRITERIOTAREFACHK: valor,
        DSCRITERIOTAREFA: valor,
      });
      this.fecharModal();
    }
  }

  private conjuntoDatas: any = [];

  public async botaoCriarTarefa() {
    const dias = Metadata.getInstance<CheckboxMultiple>('checkboxMultipleDate').value.toString();
    // Variavel onde as datas irão ficar salva no formato de string
    const datas: IDictionary = [];
    // Pegando os valores de data do campo ZdDate
    let data = Metadata.getInstance<DateInput>('selectDataDe').value;
    let data2 = Metadata.getInstance<DateInput>('selectDataAte').value;
    
    if (((data != null) ?  data.length === 10 : false) && Metadata.getInstance<Checkbox>('CheckBoxRecorrencia').value) {
      const dataCompleta = new Date(data)
      dataCompleta.setMonth(dataCompleta.getMonth() + 1);
      dataCompleta.setDate(0);
      data2 = (("0" + (dataCompleta.getMonth() + 1)).substr(-2) + "/" + ("0" + dataCompleta.getDate()).substr(-2) + "/" + dataCompleta.getFullYear());
    }
    // Criando uma validação se a data esta preenchida com todos os digitos
    
    // Verificando se a data final e igual ou superior a inicial
    let campoDataValido = ((data != null) ?  data.length === 10 : false) && ((data2 != null) ?  data2.length === 10 : false);
    if (campoDataValido) {
      data = new Date(data);
      data2 = new Date(data2);
      campoDataValido = data <= data2;
    }

    // Validando todos os campos forms da tela e criando a variável com o estado da validação
    const form = Metadata.getInstance<Form>('FormularioPrincipal').validate();
    const form1 = Metadata.getInstance<Form>('FormComHorarioEMedia').validate();

    // Instanciando uma variavel com o campo do checkbox de datas
    if (!(form && form1)) {
      this.msg('Preencha todos os campos obrigatórios (*)','error')
    } else if (!campoDataValido) {
      this.msg('Insira um período válido','info')
    } else if (dias.length === 0) {
      this.msg('Selecione o(s) dia(s) da semana','info');
    } else {
      // loop para percorrer as datas do período
      for (let dataAux = data; data <= data2; dataAux.setDate(dataAux.getDate() + 1)) {
        // Validação se o dia da data e compativel com os "Dias da semana" marcados
        if (dias.includes(dataAux.getDay())) {
          const formComHorarios = Metadata.getInstance<Form>('FormComHorarioEMedia');
          const pos = ((formComHorarios.children.length - 1) / 3);
          const func = Metadata.getInstance<Col>('colSelectFuncionario');

          for (let i = 1; i <= pos; i++) {
            const fcn = (func.children.length > 1) ? i : 1; 
            // Se a data iterada for compativel com o dia da semana ele vai salvar essa data em uma arry no formato string
            const novasDatas = new Date(`${dataAux.toJSON().substr(0,10)} ${Metadata.getInstance<Time>(`selectHoraInicio${i}`).value}`);
            const duracao = Metadata.getInstance<TextInput>(`selectTempoMedio${i}`).value;
            datas.push({ 
              DATAINICIAL: this.dataFormat(novasDatas),
              DATAFINAL: this.dataFormat(new Date(novasDatas.setMinutes(novasDatas.getMinutes() + this.calculaMinuto(duracao)))),
              NRFUNCIONARIO: Metadata.getInstance<Select>(`selectFuncionario${fcn}`).value,
              NRENCARREGADO: Metadata.getInstance<Select>(`selectFuncionarioVistoria${fcn}`).value,
              VISTORIAAUTOMATICA: Metadata.getInstance<Switch>(`switchVistoriaAways${i}`).value,
            });
          };
        }
      }
      /* Essa validação e para caso a pessoa tenha escolhido um periodo valido mas o dia escolhido não se encontra 
      nesse intervalo de datas */
      if (datas.length === 0) {
        // Exibe a msg com o erro
        this.msg('O periodo selecionado não possui o(s) dia(s) da semana escolhido.','warning');
      }
    }
    //Condicional para mandar os dados para o backend
    // Nessa condicional ele faz a validação de todos os forms estão preenchidos, e verifica se alguma data foi salva.
    if (form && form1 && datas.length > 0) {
      this.conjuntoDatas = datas;
      const func = Metadata.getInstance<Col>('colSelectFuncionario');
      const funcRev = Metadata.getInstance<Col>('colFuncionariosDaVistoria');

      let funValid = true;
      for (let i = 1; i <= func.children.length; i++) {
        if (Metadata.getInstance<Select>(`selectFuncionario${i}`).value === null) {
          funValid = false;
          break;
        };
      }

      let funcRevValid = true;
      for (let i = 1; i <= funcRev.children.length; i++) {
        if (Metadata.getInstance<Select>(`selectFuncionarioVistoria${i}`).value === null) {
          funcRevValid = false;
          break;
        };
      }

      if (!funValid && !funcRevValid){
        this.mensagem('Você esta criando uma(s) tarefa(s) sem vincular um funcionário e um encarregado, deseja continuar?', datas);
      } else if(funValid && !funcRevValid) {
        this.mensagem('Você esta criando uma(s) tarefa(s) sem vincular um encarregado, deseja continuar?', datas);
        } else if (!funValid && funcRevValid) {
          this.mensagem('Você esta criando uma(s) tarefa(s) sem vincular um funcionário, deseja continuar?', datas);
        }
        else {
        this.postTarefa(datas);
      }
    }
  }

  public criaNovoHorario(n: number) {
    return [
      {
         name:`selectHoraInicio${n}`,
         component:'ZdTime',
         autocomplete :false,
         cssClass: 'spacing-left-selects',
         label:'Inicio',
         timeFormat:'24hr',
         placeholder:'00:00',
         mask:'##:##',
         validations:{
            required:{

            }
         },
         grid:{
            cols:5
         }
      },
      {
         name:`selectTempoMedio${n}`,
         component:'ZdTextInput',
         label:'Duração (minutos)',
         cssClass: 'spacing-left-selects',
         placeholder:'00:00',
         validations:{
            required:{
               message:'Campo Obrigatorio'
            },
            minLength:{
               limit:5,
               message:'Campo Obrigatorio'
            },
            maxLength:{
               limit:5,
               message:'Campo Obrigatorio'
            }
         },
         grid:{
            cols:5
         },
      },
      {
        name: `TagSpan${n}`,
        component: 'ZdTag',
        tag: 'span',
        grid: {
          cols: 2,
        },
        children: [
          {
            name: `switchVistoriaAways${n}`,
            component: 'ZdSwitch',
            cssClass: 'switchFunionario',
            value: true,
          },
          {
            name: `text${n}`,
            component: 'ZdText',
            text: 'Criar vistoria',
            cssClass: 'textVistoria',
          },
        ],
      },
      {
        name:`selectFuncionario${n}`,
        component:"ZdSelect",
        autocomplete:false,
        cssClass: "spacing-left-selects zd-pt-6",
        label:"Funcionário",
        dataText:"name",
        dataValue:"value",
        events:{
          onBeforeMount:"{{CadastroController.setFuncionario}}"
        },
        datasource:{
          loadAll:true,
          data:[]
        },
        grid:{
          cols:12
        }
      },
      {
        name:`selectFuncionarioVistoria${n}`,
        component:"ZdSelect",
        cssClass: "zd-pt-6",
        autocomplete:false,
        label:"Encarregado",
        dataText:"name",
        dataValue:"value",
        events:{
          onBeforeMount:"{{CadastroController.setSupervisor}}"
        },
        datasource:{
          loadAll:true,
          data:[]
        },
        grid:{
          cols:12
        }
      }
    ];
  }

  public async novoHorario() {
    Metadata.getInstance<Button>('RemoveNovoHorario').isVisible = true;
    const form = Metadata.getInstance<Form>('FormComHorarioEMedia');

    const pos = ((form.children.length -1 ) / 3) + 1;

    await form.children.splice(form.children.length - 1, 0, this.criaNovoHorario(pos)[0] as IComponentRender);
    await form.children.splice(form.children.length - 1, 0, this.criaNovoHorario(pos)[1] as IComponentRender);
    await form.children.splice(form.children.length - 1, 0, this.criaNovoHorario(pos)[2] as IComponentRender);

    Metadata.getInstance<Time>(`selectHoraInicio${pos}`).value = '';
    Metadata.getInstance<Time>(`selectTempoMedio${pos}`).value = '';

    const switchFunionario = Metadata.getInstance<Switch>('switchFunionario');
    const func = Metadata.getInstance<Col>('colSelectFuncionario');
    const funcRev = Metadata.getInstance<Col>('colFuncionariosDaVistoria');
    if (!switchFunionario.value) {
      await func.children.push(this.criaNovoHorario(func.children.length + 1)[3] as IComponentRender);
      await funcRev.children.push(this.criaNovoHorario(funcRev.children.length + 1)[4] as IComponentRender);

      Metadata.getInstance<Time>(`selectFuncionario${func.children.length}`).value = '';
      Metadata.getInstance<Time>(`selectFuncionarioVistoria${func.children.length}`).value = '';
    } else {
      switchFunionario.cssStyle = {
        'margin-top': '1.5em'
      };
    }
    switchFunionario.readonly = true;
  }

  public removeHorario() {
    const form = Metadata.getInstance<Form>('FormComHorarioEMedia');
    const func = Metadata.getInstance<Col>('colSelectFuncionario');
    const funcRev = Metadata.getInstance<Col>('colFuncionariosDaVistoria');

    const switchFunionario = Metadata.getInstance<Switch>('switchFunionario');

    if (form.children.length > 4) {
      if (!switchFunionario.value){
        func.children.splice(func.children.length - 1, 1);
        funcRev.children.splice(funcRev.children.length - 1, 1);
      }

      form.children.splice(form.children.length - 4, 3);
      if (form.children.length === 4) {
        Metadata.getInstance<Button>('RemoveNovoHorario').isVisible = false;
        switchFunionario.cssStyle = '';
        switchFunionario.readonly = false;
      }
    }
  }

  public dataFormat(data: Date) {
        const dia  = data.getDate().toString();
        const diaF = (dia.length == 1) ? '0'+dia : dia;
        const mes  = (data.getMonth()+1).toString(); //+1 pois no getMonth Janeiro começa com zero.
        const mesF = (mes.length == 1) ? '0'+mes : mes;
        const anoF = data.getFullYear();
        const hora = data.getHours().toString();
        const horaF = (hora.length == 1) ? '0'+hora : hora;
        const minuto = data.getMinutes().toString();
        const minutoF = (minuto.length == 1) ? '0'+minuto : minuto;
    return anoF+"/"+mesF+"/"+diaF+" "+horaF+":"+minutoF;
  }

  public msg(text: string,tipo: string) {
    AlertService.show({
        name: 'basic-alert',
        text: text,
        top: true,
        left: false,
        timeout: 3000,
        color: tipo,
      });
  }

  public mensagem (texto:  string, datas: IDictionary) {
  	DialogService.show({
  	  name: 'basic-dialog',
  	  type: 'warning',
  	  title: 'Atenção!',
  	  text: `${texto}`,
  	  buttons: [
      {
  		  name: 'cancel',
  		  component: 'ZdButton',
  		  outline: true,
  		  label: 'Voltar',
        events: { 
  		   click: () => DialogService.hide(),
  		  }
  		},
  		{
  		  name: 'yes',
  		  component: 'ZdButton',
  		  label: 'Criar',
  		  events: {
  		   click: () => this.postTarefa(datas),
  		  },
  		},
  	  ],
  	});
  }

  public postTarefa(datas: IDictionary) {
    DialogService.hide();
    
    const tarefasSelecionas = Metadata.getInstance<Select>('selectTarefa').value.flatMap((value: string) => (String(value)).split(','));
    LoadingService.show();
    Http.post('/insertNovaTarefa', {
    	NMTAREFA: Metadata.getInstance<TextInput>('setNomeTarefa').value,
    	LOCAL: Metadata.getInstance<SelectMultiple>('selectNomeLocal').value,
    	CHECKLIST: Metadata.getInstance<Select>('selectChecklist').value,
    	ITENSCHECK: Metadata.getInstance<Select>('selectTipoTarefa').value,
    	TAREFA: tarefasSelecionas,
    	CRITERIOS: Metadata.getInstance<CheckboxMultiple>('CheckBoxDeCriterios').value,
    	DATA: this.conjuntoDatas,
    	RECORRENTE: Metadata.getInstance<Checkbox>('CheckBoxRecorrencia').value,
    	CDOPERADOR: localStorage.getItem('CDOPERADOR'),
    	DIASSEMANA: Metadata.getInstance<CheckboxMultiple>('checkboxMultipleDate').value.toString(),
    	HORAS: this.getHours(),
    }).then((response: IDictionary) => {
      Utilities.showSuccessMessage();
       Router.replace(this.origem);
    }).catch((error: any) => {
      Utilities.showErrorMessage(error);
    }).finally(() => {
      LoadingService.hide();
    });
  }
  
  public calculaMinuto(minutos: string) {
    let tempoTotal = (Math.floor(Number(minutos.replace(':', '')) / 100)) * 60;
    tempoTotal += Math.floor(Number(minutos.replace(':', '')) % 100);
    return tempoTotal;
  }
  
  public getHours() {
    const pos = ((Metadata.getInstance<Form>('FormComHorarioEMedia').children.length - 1) / 3);
    const horas: any = [];
    const func = Metadata.getInstance<Col>('colSelectFuncionario');
    for (let i = 1; i <= pos; i++) {
      const fcn = (func.children.length > 1) ? i : 1;
      horas.push({ 
        HORARIO: Metadata.getInstance<Time>(`selectHoraInicio${i}`).value,
        DURACAO: this.calculaMinuto(Metadata.getInstance<TextInput>(`selectTempoMedio${i}`).value),
        NRFUNCIONARIO: Metadata.getInstance<Select>(`selectFuncionario${fcn}`).value,
        NRENCARREGADO: Metadata.getInstance<Select>(`selectFuncionarioVistoria${fcn}`).value,
        VISTORIAAUTOMATICA: Metadata.getInstance<Switch>(`switchVistoriaAways${i}`).value,
      });
    };
    return horas;
  }
}