/* eslint-disable */
import {
  Http, IDictionary, Metadata,
} from '@zeedhi/core';
import { 
  Modal, ModalService, Select,
  TextInput,
} from '@zeedhi/common';

export class ModalTarefasSolicitacoesController {
  private classifyModal!: Modal;

  private insertTypeModal!: Modal;

  /* -------- Criar Modais -------- */

  public async createClassifyModal() {
    this.classifyModal = await ModalService.createFromJson('/modal/modalTarefasClassificarSolicitacao', true);
  }

  public async createInsertTypeModal() {
    this.insertTypeModal = await ModalService.createFromJson('/modal/modalTarefasTipoSolicitacao', true);
  }

  /* -------- Destruir Modais -------- */

  public destroyClassifyModal() {
    this.classifyModal.destroy();
  }

  public destroyInsertTypeModal() {
    this.insertTypeModal.destroy();
  }

  /* -------- Abrir Modais -------- */

  public openClassifyModal() {
    this.classifyModal.show();
  }

  public openInsertTypeModal() {
    this.insertTypeModal.show();
  }

  /* -------- Fechar Modais -------- */

  public closeClassifyModal() {
    this.classifyModal.hide();
  }

  public closeInsertTypeModal() {
    this.insertTypeModal.hide();
  }
  
  /* ---------------modal : Classificar Solicitação --------------- */
  
  public initializeData() {
    this.getEmployees();
    this.getRequestTypes();
    this.getChecklist();
  }
  
  // Busca e adiciona ao select de responsável da tarefa
  public getEmployees() {
    const taskResponsibleSelect = Metadata.getInstance<Select>('classifyResponsible');
    
    Http.get('/usersList').then((response: IDictionary) => {
      response.data.dataset.info.forEach((database: IDictionary) => {
        taskResponsibleSelect.datasource.data.push({
          value: database.NRFUNCIONARIO,
          name: database.NMFUNCIONARIO
        });
      })
    })
  
    
  }
  
  // Busca os tipos de requisição e monta o select
  public getRequestTypes() {
    const selectObject = Metadata.getInstance<Select>('classifyType');
    Http.get('/getRequestTypes').then((response: IDictionary) => {
      response.data.dataset.info.forEach((database: IDictionary) => {
        selectObject.datasource.data.push({
          value: database.NRTIPOSOLICITACAO,
          name: database.NMTIPOSOLICITACAO,
          priority: database.IDPRIORIDADEPADR,
        });
      });
    });
  }
  
  // Busca os checklists e monta o select
  public getChecklist() {
    const selectObject = Metadata.getInstance<Select>('classifyChecklist');

    Http.get('/getCheckListLocalAtend').then((response: IDictionary) => {
      response.data.dataset.options.forEach((database: IDictionary) => {
        selectObject.datasource.data.push({
          name: database.NMCHECKLIST, 
          value: database.NRCHECKLIST,
        });
      });
    });
  }
}
